@@ -37,10 +37,18 @@ requests_cache.core.install_cache(backend='memory', expire_after=30.0, fast_save
 import lxml, lxml.objectify
 import dateutil, dateutil.parser
 
+#monkeypatch urllib3 param function to bypass bug in jottacloud servers
+from requests.packages import urllib3
+urllib3.fields.format_header_param_orig = urllib3.fields.format_header_param
+def mp(name, value):
+    return urllib3.fields.format_header_param_orig(name, value).replace('filename*=', 'filename=')
+urllib3.fields.format_header_param = mp
+
+
+
 # some setup
 JFS_ROOT='https://www.jotta.no/jfs/'
 JFS_CACHELIMIT=1024*1024 # stuff below this threshold (in bytes) will be cached
-logging.basicConfig(level=logging.DEBUG)
 
 class JFSError(Exception):
     @staticmethod
@@ -150,10 +158,10 @@ class JFSFolder(object):
     def up(self, fileobj_or_path, filename=None):
         'Upload a file to current folder and return the new JFSFile'
         if not isinstance(fileobj_or_path, file):
+            filename = os.path.basename(fileobj_or_path)
             fileobj_or_path = open(fileobj_or_path, 'rb')
-        if filename is None:
-            filename = os.path.basename(fileobj_or_path.name)
-        r = self.jfs.up('%s/%s' % (self.path, filename), fileobj_or_path)
+        logging.debug('.up %s ->  %s %s', repr(fileobj_or_path), repr(self.path), repr(filename))
+        r = self.jfs.up(os.path.join(self.path, filename), fileobj_or_path)
         self.sync()
         return r
 
@@ -209,6 +217,12 @@ class JFSFile(object):
                 updated = '2010-11-19-T12:34:28Z' [StringElement]
         """
 
+    def readpartial(self, start, end):
+        'Get a part of the file, from start byte to end byte (integers)'
+        return self.jfs.raw('%s?mode=bin' % self.path,
+                            usecache=False,
+                            extra_headers={'Range':'bytes=%s-%s' % (start, end)})
+
     def write(self, data):
         'Put, possibly replace, file contents with (new) data'
 #         return self.jfs.post(self.path, content=data,
@@ -492,29 +506,32 @@ class JFS(object):
         self.username = username
         self.session.auth = HTTPBasicAuth(username, password)
         self.session.verify = ca_bundle
-        self.session.headers =  {'User-Agent':'JottaFS %s (https://gitorious.org/jottafs/)' % (__version__, ),
+        self.session.headers =  {'User-Agent':'jottalib %s (https://gitorious.org/jottafs/jottalib)' % (__version__, ),
                                  'X-JottaAPIVersion': self.apiversion,
                                 }
         self.rootpath = JFS_ROOT + username
         self.fs = self.get(self.rootpath)
 
-    def request(self, url, usecache=True):
+    def request(self, url, usecache=True, extra_headers=None):
+        'Make a GET request for url, with or without caching'
         if not url.startswith('http'):
             # relative url
             url = self.rootpath + url
-        logging.debug("getting url: %s" % url)
+        logging.debug("getting url: %s, usecache=%s, extra_headers=%s", url, usecache, extra_headers)
+        if extra_headers is None: extra_headers={}
         if usecache:
-            r = self.session.get(url)
+            r = self.session.get(url, headers=extra_headers)
         else:
             with requests_cache.disabled():
-                r = self.session.get(url)
+                r = self.session.get(url, headers=extra_headers)
 
         if r.status_code in ( 500, ):
             raise JFSError(r.reason)
         return r
 
-    def raw(self, url, usecache=True):
-        r = self.request(url, usecache)
+    def raw(self, url, usecache=True, extra_headers=None):
+        'Make a GET request for url and return whatever content we get'
+        r = self.request(url, usecache=usecache, extra_headers=extra_headers)
         # uncomment to dump raw xml
         # f = open('/tmp/%s.xml' % time.time(), 'wb')
         # f.write(r.content)
@@ -522,13 +539,14 @@ class JFS(object):
         return r.content
 
     def get(self, url, usecache=True):
+        'Make a GET request for url and return the response content as a generic lxml object'
         o = lxml.objectify.fromstring(self.raw(url, usecache=usecache))
         if o.tag == 'error':
             JFSError.raiseError(o, url)
         return o
 
     def getObject(self, url_or_requests_response, usecache=True):
-        'Take a url or some xml response from JottaCloud and match it up to one of our classes'
+        'Take a url or some xml response from JottaCloud and wrap it up with the corresponding JFS* class'
         if isinstance(url_or_requests_response, requests.models.Response):
             url = url_or_requests_response.url
             o = lxml.objectify.fromstring(url_or_requests_response.content)
@@ -573,7 +591,7 @@ class JFS(object):
         return self.getObject(r) # return a JFS* class
 
     def up(self, path, fileobject):
-        "Upload a fileobject to path, HTTP POST-ing to up.jottacloud.com"
+        "Upload a fileobject to path, HTTP POST-ing to up.jottacloud.com, using the JottaCloud API"
         """
 
         *** WHAT DID I DO?: created file
@@ -600,9 +618,9 @@ class JFS(object):
         url = path.replace('www.jotta.no', 'up.jottacloud.com')
         content = fileobject.read()
         fileobject.seek(0) # rewind read index for requests.post
-        md5hash = hashlib.md5(content).hexdigest()
+        md5hash = hashlib.md5(content).hexdigest() # TODO: read (big) files in chunks to avoid memory errors
         logging.debug('posting content (len %s, hash %s) to url %s', len(content), md5hash, url)
-        now = '2014-10-05T10:23:18Z+00:00' #datetime.datetime.now().isoformat() # TODO: don't hardcode
+        now = datetime.datetime.now().isoformat()
         headers = {'JMd5':md5hash,
                    'JCreated': now,
                    'JModified': now,
@@ -611,6 +629,7 @@ class JFS(object):
                    'jx_csid': '',
                    'jx_lisence': ''
                    }
+        #TODO: enable partial uploading using HTTP Range bytes=start
         params = {'cphash':md5hash,}
         files = {'md5': ('', md5hash),
                  'modified': ('', now),
@@ -622,7 +641,7 @@ class JFS(object):
     @property
     def devices(self):
         'return generator of configured devices'
-        return self.fs is not None and [JFSDevice(d, self, parentpath=self.path) for d in self.fs.devices.iterchildren()] or [x for x in []]
+        return self.fs is not None and [JFSDevice(d, self, parentpath=self.rootpath) for d in self.fs.devices.iterchildren()] or [x for x in []]
 
     @property
     def locked(self):
@@ -653,17 +672,31 @@ class JFS(object):
 if __name__=='__main__':
     # debug setup
     import httplib as http_client
-    http_client.HTTPConnection.debuglevel = 1
-    requests_log = logging.getLogger("requests.packages.urllib3")
-    requests_log.setLevel(logging.DEBUG)
-    requests_log.propagate = True
+    logging.basicConfig(level=logging.DEBUG)
+    #http_client.HTTPConnection.debuglevel = 1
+    #requests_log = logging.getLogger("requests.packages.urllib3")
+    #requests_log.setLevel(logging.DEBUG)
+    #requests_log.propagate = True
     from lxml.objectify import dump as xdump
     from pprint import pprint
-    jfs = JFS(os.environ['JOTTACLOUD_USERNAME'], password=os.environ['JOTTACLOUD_PASSWORD'])
+    import netrc
+    try:
+        n = netrc.netrc()
+        username, account, password = n.authenticators('jottacloud') # read .netrc entry for 'machine jottacloud'
+    except Exception as e:
+        logging.exception(e)
+        username = os.environ['JOTTACLOUD_USERNAME']
+        password = os.environ['JOTTACLOUD_PASSWORD']
+
+    jfs = JFS(username, password)
     #logging.info(xdump(jfs.fs))
     jottadev = None
     for j in jfs.devices:
         if j.name == 'Jotta':
             jottadev = j
     jottasync = jottadev.mountPoints['Sync']
-    r = jottasync.up('/tmp/test.pdf')
+    try:
+        _filename = sys.argv[1]
+    except IndexError:
+        _filename = '/tmp/test.pdf'
+    r = jottasync.up(_filename)
